@@ -16,7 +16,9 @@ import json
 import os
 
 from flask import Flask
-from loguru import logger
+import logging
+
+logger = logging.getLogger(__name__)
 import pydicom
 
 from pymedphys._dicom.connect.listen import DicomListener
@@ -79,7 +81,7 @@ class FlaskApp(Flask):
             **options
         )
 
-    def run_dicom_listener(self, listen_port, listen_ae_title):
+    def run_dicom_listener(self, listen_port=None, listen_ae_title=None):
         """
         Background task that listens at a specific port for incoming dicom series
         """
@@ -87,14 +89,20 @@ class FlaskApp(Flask):
         from .models import Dataset, DataObject
         from . import db
 
+        if listen_port is None:
+            listen_port = self.dicom_listener_port
+
+        if listen_ae_title is None:
+            listen_ae_title = self.dicom_listener_aetitle
+
         logger.info(
-            "Starting Dicom Listener on port: {0} with AE Title: {1}",
+            "Starting Dicom Listener on port: %s with AE Title: %s",
             listen_port,
             listen_ae_title,
         )
 
         def series_recieved(dicom_path):
-            logger.info("Series Recieved at path: {0}".format(dicom_path))
+            logger.info("Series Recieved at path: %s", dicom_path)
 
             # Get the SeriesUID
             series_uid = None
@@ -105,11 +113,11 @@ class FlaskApp(Flask):
                     d = pydicom.read_file(f)
                     series_uid = d.SeriesInstanceUID
                 except Exception as e:
-                    logger.debug("No Series UID in: {0}".format(f))
+                    logger.debug("No Series UID in: %s", f)
                     logger.debug(e)
 
             if series_uid:
-                logger.info("Image Series UID: {0}".format(series_uid))
+                logger.info("Image Series UID: %s", series_uid)
             else:
                 logger.error("Series UID could not be determined... Stopping")
                 return
@@ -118,23 +126,24 @@ class FlaskApp(Flask):
             dos = DataObject.query.filter_by(series_instance_uid=series_uid).all()
 
             if len(dos) == 0:
-                logger.error(
-                    "No Data Object found with Series UID: {0} ... Stopping".format(series_uid)
-                )
+                logger.error("No Data Object found with Series UID: %s ... Stopping", series_uid)
                 return
 
             for do in dos:
 
                 do.is_fetched = True
-                do.path = dicom_path
+                do.path = str(dicom_path)
                 db.session.commit()
 
         try:
             dicom_listener = DicomListener(
-                port=listen_port, ae_title=listen_ae_title, on_released_callback=series_recieved
+                host="0.0.0.0",
+                port=listen_port,
+                ae_title=listen_ae_title,
+                on_released_callback=series_recieved,
             )
 
             dicom_listener.start()
 
         except Exception as e:
-            logger.error("Listener Error: " + str(e))
+            logger.error("Listener Error: %s", e)
