@@ -16,7 +16,9 @@
 
 import sys
 
-from loguru import logger
+import logging
+
+logger = logging.getLogger(__name__)
 
 from platipy.cli import (
     dicom_crawler,
@@ -49,10 +51,6 @@ except ImportError:
     )
 
 
-logger.remove()
-logger.add(sys.stderr, level="DEBUG")
-
-
 def platipy_cli():
     """Run the PlatiPy Command Line Interface"""
     if len(sys.argv) == 1 or not sys.argv[1] in tools:
