@@ -49,8 +49,8 @@ def calculate_dvh(dose_grid, label, bins=1001):
     values = np.cumsum(counts[::-1])[::-1]
     if np.all(values == 0):
         return bins, values
-    else:
-        values = values / values.max()
+
+    values = values / values.max()
 
     return bins, values
 
@@ -106,7 +106,7 @@ def calculate_dvh_for_labels(dose_grid, labels, bin_width=0.1, max_dose=None):
                 "cc": cc,
                 "mean": mean_dose,
             },
-            **{d: c for d, c in zip(bins, values)},
+            **dict(zip(bins, values)),
         }
 
         dvh.append(entry)
@@ -119,7 +119,8 @@ def calculate_d_x(dvh, x, label=None):
 
     Args:
         dvh (pandas.DataFrame): DVH DataFrame as produced by calculate_dvh_for_labels
-        x (float): The dose which x percent of the volume receives
+        x (float|list): The dose threshold (or list of dose thresholds) which x percent of the
+            volume receives
         label (str, optional): The label to compute the metric for. Computes for all metrics if not
             set. Defaults to None.
 
@@ -130,15 +131,31 @@ def calculate_d_x(dvh, x, label=None):
     if label:
         dvh = dvh[dvh.label == label]
 
+    if not isinstance(x, list):
+        x = [x]
+
     bins = np.array([b for b in dvh.columns if isinstance(b, float)])
     values = np.array(dvh[bins])
 
-    i, j = np.where(values >= x / 100)
-
     metrics = []
     for idx in range(len(dvh)):
         d = dvh.iloc[idx]
-        metrics.append({"label": d.label, "metric": f"D{x}", "value": bins[j][i == idx][-1]})
+
+        m = {"label": d.label}
+
+        for threshold in x:
+            value = np.interp(threshold / 100, values[idx][::-1], bins[::-1])
+            if values[idx, 0] == np.sum(values[idx]):
+                value = 0
+
+            # Interp will return zero when computing D100, do compute this separately
+            if threshold == 100:
+                i, j = np.where(values == 1.0)
+                value = bins[j][i == idx][-1]
+
+            m[f"D{threshold}"] = value
+
+        metrics.append(m)
 
     return pd.DataFrame(metrics)
 
@@ -148,7 +165,7 @@ def calculate_v_x(dvh, x, label=None):
 
     Args:
         dvh (pandas.DataFrame): DVH DataFrame as produced by calculate_dvh_for_labels
-        x (float): The dose to get the volume for.
+        x (float|list): The dose threshold (or list of dose thresholds) to get the volume for.
         label (str, optional): The label to compute the metric for. Computes for all metrics if not
             set. Defaults to None.
 
@@ -159,21 +176,64 @@ def calculate_v_x(dvh, x, label=None):
     if label:
         dvh = dvh[dvh.label == label]
 
+    if not isinstance(x, list):
+        x = [x]
+
     bins = np.array([b for b in dvh.columns if isinstance(b, float)])
     values = np.array(dvh[bins])
 
-    i = np.where(bins == x)
     metrics = []
     for idx in range(len(dvh)):
         d = dvh.iloc[idx]
-        value_idx = values[idx, i]
-        value = 0.0
-        if value_idx.shape[1] > 0:
-            value = d.cc * values[idx, i][0, 0]
 
-        metric_name = f"V{x}"
-        if x - int(x) == 0:
-            metric_name = f"V{int(x)}"
-        metrics.append({"label": d.label, "metric": metric_name, "value": value})
+        m = {"label": d.label}
+
+        for threshold in x:
+            value = np.interp(threshold, bins, values[idx]) * d.cc
+
+            metric_name = f"V{threshold}"
+            if threshold - int(threshold) == 0:
+                metric_name = f"V{int(threshold)}"
+
+            m[metric_name] = value
+
+        metrics.append(m)
+
+    return pd.DataFrame(metrics)
+
+
+def calculate_d_cc_x(dvh, x, label=None):
+    """Compute the dose which is received by cc of the volume
+
+    Args:
+        dvh (pandas.DataFrame): DVH DataFrame as produced by calculate_dvh_for_labels
+        x (float|list): The cc (or list of cc's) to compute the dose at.
+        label (str, optional): The label to compute the metric for. Computes for all metrics if not
+            set. Defaults to None.
+
+    Returns:
+        pandas.DataFrame: Data frame with a row for each label containing the metric and value.
+    """
+
+    if label:
+        dvh = dvh[dvh.label == label]
+
+    if not isinstance(x, list):
+        x = [x]
+
+    metrics = []
+    for idx in range(len(dvh)):
+
+        d = dvh.iloc[idx]
+        m = {"label": d.label}
+
+        for threshold in x:
+            cc_at = (threshold / dvh[dvh.label == d.label].cc.iloc[0]) * 100
+            cc_at = min(cc_at, 100)
+            cc_val = calculate_d_x(dvh[dvh.label == d.label], cc_at)[f"D{cc_at}"].iloc[0]
+
+            m[f"D{threshold}cc"] = cc_val
+
+        metrics.append(m)
 
     return pd.DataFrame(metrics)
