@@ -13,16 +13,15 @@
 # limitations under the License.
 
 import warnings
+import logging
 
 import matplotlib.pyplot as plt
 import matplotlib.gridspec as gridspec
-from mpl_toolkits.axes_grid1 import make_axes_locatable  # , AxesGrid, ImageGrid
+#from mpl_toolkits.axes_grid1 import make_axes_locatable  # , AxesGrid, ImageGrid
 
 import numpy as np
 import SimpleITK as sitk
 
-from loguru import logger
-
 from platipy.imaging.utils.crop import label_to_roi
 
 from platipy.imaging.visualisation.utils import (
@@ -39,6 +38,8 @@ from platipy.imaging.visualisation.utils import (
     project_onto_arbitrary_plane,
 )
 
+logger = logging.getLogger(__name__)
+
 """
 This Python script comprises two contributions to the code base:
 1) A bunch of helpful visualisation "helper" functions
@@ -108,12 +109,13 @@ class ImageVisualiser:
         self.__comparison_overlays = []
         self.__vector_overlays = []
 
-    def set_limits_from_label(self, label, expansion=[0, 0, 0]):
+    def set_limits_from_label(self, label, expansion=2):
         """Sets the limits of the axes to the bounds of the given label.
 
         Args:
             label (sitk.Image): The label around which to set the limits
-            expansion (list, optional): Expansion (in mm) around the label. Defaults to [0, 0, 0].
+            expansion (list | float, optional): Expansion (in mm) around the label.
+                Defaults to 2.
         """
 
         (sag_size, cor_size, ax_size), (sag_0, cor_0, ax_0) = label_to_roi(
@@ -295,6 +297,8 @@ class ImageVisualiser:
         min_value=False,
         max_value=False,
         colormap=plt.cm.get_cmap("inferno"),
+        discrete_levels=False,
+        mid_ticks=False,
         alpha=0.75,
         arrow_scale=1,
         arrow_width=1,
@@ -341,6 +345,8 @@ class ImageVisualiser:
                 min_value=min_value,
                 max_value=max_value,
                 colormap=colormap,
+                discrete_levels=discrete_levels,
+                mid_ticks=mid_ticks,
                 alpha=alpha,
                 arrow_scale=arrow_scale,
                 arrow_width=arrow_width,
@@ -479,8 +485,8 @@ class ImageVisualiser:
                 window = (lower, upper - lower)
         try:
             logger.info(
-                f"Found a (z,y,x,{nda.shape[3]}) dimensional array - assuming this is an RGB"
-                "image."
+                "Found a (z,y,x,%s) dimensional array - assuming this is an RGB"
+                "image.", nda.shape[3]
             )
             nda /= nda.max()
         except ValueError:
@@ -491,6 +497,14 @@ class ImageVisualiser:
         sp_plane, _, sp_slice = image.GetSpacing()
         asp = (1.0 * sp_slice) / sp_plane
 
+        # Get the size - this is used for extent
+        size_sag, size_cor, size_ax = self.__image.GetSize()
+        extent_dict = {
+            "x": (0, size_cor, 0, size_ax),
+            "y": (0, size_sag, 0, size_ax),
+            "z": (0, size_sag, 0, size_cor),
+        }
+
         if self.__projection is True:
             projection = "max"
         else:
@@ -564,6 +578,7 @@ class ImageVisualiser:
                 origin={"normal": "upper", "reversed": "lower"}[self.__origin],
                 cmap=self.__colormap,
                 clim=(window[0], window[0] + window[1]),
+                extent=extent_dict["z"],
             )
             cor_view = ax_cor.imshow(
                 cor_img,
@@ -572,6 +587,7 @@ class ImageVisualiser:
                 interpolation="none",
                 cmap=self.__colormap,
                 clim=(window[0], window[0] + window[1]),
+                extent=extent_dict["y"],
             )
             sag_view = ax_sag.imshow(
                 sag_img,
@@ -580,6 +596,7 @@ class ImageVisualiser:
                 interpolation="none",
                 cmap=self.__colormap,
                 clim=(window[0], window[0] + window[1]),
+                extent=extent_dict["x"],
             )
 
             ax_ax.axis("off")
@@ -660,6 +677,7 @@ class ImageVisualiser:
                 origin=org,
                 cmap=self.__colormap,
                 clim=(window[0], window[0] + window[1]),
+                extent=extent_dict[self.__axis],
             )
             ax.axis("off")
 
@@ -708,6 +726,14 @@ class ImageVisualiser:
                 upper = np.percentile(nda_original, 99)
                 window = (lower, upper - lower)
 
+        # Get the size - this is used for extent
+        size_sag, size_cor, size_ax = self.__image.GetSize()
+        extent_dict = {
+            "x": (0, size_cor, 0, size_ax),
+            "y": (0, size_sag, 0, size_ax),
+            "z": (0, size_sag, 0, size_cor),
+        }
+
         if self.__axis == "ortho":
             figure_size = (
                 self.__figure_size,
@@ -748,6 +774,7 @@ class ImageVisualiser:
                 aspect=1.0,
                 origin={"normal": "upper", "reversed": "lower"}[self.__origin],
                 interpolation="none",
+                extent=extent_dict["z"],
             )
 
             nda_colormix = generate_comparison_colormix(
@@ -762,6 +789,7 @@ class ImageVisualiser:
                 origin="lower",
                 aspect=asp,
                 interpolation="none",
+                extent=extent_dict["y"],
             )
 
             nda_colormix = generate_comparison_colormix(
@@ -776,6 +804,7 @@ class ImageVisualiser:
                 origin="lower",
                 aspect=asp,
                 interpolation="none",
+                extent=extent_dict["x"],
             )
 
             ax_ax.axis("off")
@@ -837,6 +866,7 @@ class ImageVisualiser:
                 aspect=asp,
                 interpolation="none",
                 origin=org,
+                extent=extent_dict[self.__axis],
             )
             ax.axis("off")
 
@@ -884,11 +914,12 @@ class ImageVisualiser:
                     1 / asp * (cor_orig_1 - cor_orig_0) + (ax_orig_1 - ax_orig_0)
                 )
 
-                if origin == "reversed":
-                    cor_0, cor_1 = cor_1, cor_0
-
                 ax_ax.set_xlim(sag_0, sag_1)
-                ax_ax.set_ylim(cor_1, cor_0)
+
+                if origin == "reversed":
+                    ax_ax.set_ylim(cor_0, cor_1)
+                else:
+                    ax_ax.set_ylim(cor_orig_1 - cor_1, cor_orig_1 - cor_0)
 
                 ax_cor.set_xlim(sag_0, sag_1)
                 ax_cor.set_ylim(ax_0, ax_1)
@@ -945,7 +976,7 @@ class ImageVisualiser:
                 x_0, x_1 = sorted([x_0, x_1])
                 y_0, y_1 = sorted([y_0, y_1])
 
-                if self.__axis == "z":
+                if self.__axis == "z" and self.__origin == "normal":
                     y_0, y_1 = y_1, y_0
 
                 ratio_x = np.abs(x_1 - x_0) / np.abs(x_orig_1 - x_orig_0)
@@ -990,6 +1021,14 @@ class ImageVisualiser:
         # Test types of axes
         axes = self.__figure.axes[:4]
 
+        # Get the size - this is used for extent
+        size_sag, size_cor, size_ax = self.__image.GetSize()
+        extent_dict = {
+            "x": (0, size_cor, 0, size_ax),
+            "y": (0, size_sag, 0, size_ax),
+            "z": (0, size_sag, 0, size_cor),
+        }
+
         if self.__axis in ["x", "y", "z"]:
             ax = axes[0]
             s = return_slice(self.__axis, self.__cut)
@@ -1012,6 +1051,11 @@ class ImageVisualiser:
                     )
                     contour_disp = sitk.GetArrayFromImage(contour_disp_proj)
 
+                if self.__axis == "z":
+                    origin = {"normal": "upper", "reversed": "lower"}[self.__origin]
+                else:
+                    origin = "lower"
+
                 try:
                     ax.contour(
                         contour_disp,
@@ -1020,7 +1064,8 @@ class ImageVisualiser:
                         # alpha=0.8,
                         linewidths=lw_dict[c_name],
                         linestyles=ls_dict[c_name],
-                        origin="lower",
+                        extent=extent_dict[self.__axis],
+                        origin=origin,
                     )
                     ax.plot(
                         [0],
@@ -1086,11 +1131,12 @@ class ImageVisualiser:
 
                 ax_ax.contour(
                     contour_ax,
-                    levels=[0.5],
+                    levels=[0],
                     linewidths=lw_dict[c_name],
                     linestyles=ls_dict[c_name],
                     colors=[color_dict[c_name]],
-                    origin="lower",
+                    extent=extent_dict["z"],
+                    origin={"normal": "upper", "reversed": "lower"}[self.__origin],
                 )
                 ax_ax.plot(
                     [0],
@@ -1107,6 +1153,7 @@ class ImageVisualiser:
                     linewidths=lw_dict[c_name],
                     linestyles=ls_dict[c_name],
                     colors=[color_dict[c_name]],
+                    extent=extent_dict["y"],
                     origin="lower",
                 )
                 ax_sag.contour(
@@ -1115,6 +1162,7 @@ class ImageVisualiser:
                     linewidths=lw_dict[c_name],
                     linestyles=ls_dict[c_name],
                     colors=[color_dict[c_name]],
+                    extent=extent_dict["x"],
                     origin="lower",
                 )
 
@@ -1153,10 +1201,20 @@ class ImageVisualiser:
             else:
                 norm = None
 
+            # Get the spacing
             sp_plane, _, sp_slice = scalar_image.GetSpacing()
 
+            # Get the aspect ratio
             asp = (1.0 * sp_slice) / sp_plane
 
+            # Get the size - this is used for extent
+            size_sag, size_cor, size_ax = self.__image.GetSize()
+            extent_dict = {
+                "x": (0, size_cor, 0, size_ax),
+                "y": (0, size_sag, 0, size_ax),
+                "z": (0, size_sag, 0, size_cor),
+            }
+
             # projection organisation
             if scalar.projection:
                 projection = scalar.projection
@@ -1218,6 +1276,7 @@ class ImageVisualiser:
                     vmax=s_max,
                     alpha=alpha,
                     norm=norm,
+                    extent=extent_dict["z"],
                 )
 
                 cor_view = ax_cor.imshow(
@@ -1231,6 +1290,7 @@ class ImageVisualiser:
                     vmax=s_max,
                     alpha=alpha,
                     norm=norm,
+                    extent=extent_dict["y"],
                 )
 
                 sag_view = ax_sag.imshow(
@@ -1244,6 +1304,7 @@ class ImageVisualiser:
                     vmax=s_max,
                     alpha=alpha,
                     norm=norm,
+                    extent=extent_dict["x"],
                 )
 
                 # this is for (work-in-progress) dynamic visualisation
@@ -1274,18 +1335,24 @@ class ImageVisualiser:
 
                 asp = {"x": asp, "y": asp, "z": 1}[self.__axis]
 
+                if self.__axis == "z":
+                    origin = {"normal": "upper", "reversed": "lower"}[self.__origin]
+                else:
+                    origin = "lower"
+
                 s = return_slice(self.__axis, self.__cut)
                 ax_view = ax.imshow(
                     disp_img,
                     interpolation="none",
                     cmap=colormap,
                     clim=(s_min, s_max),
-                    origin="lower",
+                    origin=origin,
                     aspect=asp,
                     vmin=s_min,
                     vmax=s_max,
                     alpha=alpha,
                     norm=norm,
+                    extent=extent_dict[self.__axis],
                 )
 
             if scalar.show_colorbar:
@@ -1362,7 +1429,7 @@ class ImageVisualiser:
         if self.__projection and len(self.__vector_overlays) > 0:
             raise Warning("Vector overlay is not implemented in projection mode.")
 
-        for _, vector in enumerate(self.__vector_overlays):
+        for vector_index, vector in enumerate(self.__vector_overlays):
 
             image = vector.image
             colormap = vector.colormap
@@ -1379,6 +1446,10 @@ class ImageVisualiser:
             inverse_vector_image = image  # sitk.InvertDisplacementField(image)
             vector_nda = sitk.GetArrayFromImage(inverse_vector_image)
 
+            if vector.discrete_levels:
+                colormap_name = vector.colormap.name
+                colormap = plt.cm.get_cmap(colormap_name, vector.discrete_levels)
+
             # Test types of axes
             axes = self.__figure.axes
             if len(axes[:4]) < 4:
@@ -1414,9 +1485,9 @@ class ImageVisualiser:
                         vector_plot_x ** 2 + vector_plot_y ** 2 + vector_plot_z ** 2
                     )
 
-                if ~max_value:
+                if max_value is False:
                     max_value = vector_color.max()
-                if ~min_value:
+                if min_value is False:
                     min_value = vector_color.min()
 
                 sp_vector = ax.quiver(
@@ -1435,20 +1506,23 @@ class ImageVisualiser:
                     clim=[min_value, max_value],
                 )
 
-                if vector.show_colorbar:
-                    divider = make_axes_locatable(ax)
-                    cax = divider.append_axes("right", size="5%", pad=0.05)
-                    cbar = self.__figure.colorbar(sp_vector, cax=cax, orientation="vertical")
-                    cbar.set_label(vector.name)
-                    cbar.solids.set_alpha(1)
+                # if vector.show_colorbar:
+                #     divider = make_axes_locatable(ax)
+                #     cax = divider.append_axes("right", size="5%", pad=0.05)
+                #     cbar = self.__figure.colorbar(sp_vector, cax=cax, orientation="vertical")
+                #     cbar.set_label(vector.name)
+                #     cbar.solids.set_alpha(1)
 
-                    f_x, f_y = self.__figure.get_size_inches()
-                    self.__figure.set_size_inches(f_x * 1.15, f_y)
-                    self.__figure.subplots_adjust(left=0, right=0.88, bottom=0, top=1)
+                #     f_x, f_y = self.__figure.get_size_inches()
+                #     self.__figure.set_size_inches(f_x * 1.15, f_y)
+                #     self.__figure.subplots_adjust(left=0, right=0.88, bottom=0, top=1)
 
             elif len(axes) >= 4:
                 ax_ax, _, ax_cor, ax_sag = axes[:4]
 
+                # set this variable for colorbar reference
+                ax = ax_ax
+
                 for plot_axes, im_axis, im_cut in zip(
                     (ax_ax, ax_cor, ax_sag), ("z", "y", "x"), self.__cut
                 ):
@@ -1482,9 +1556,9 @@ class ImageVisualiser:
                             vector_plot_x ** 2 + vector_plot_y ** 2 + vector_plot_z ** 2
                         )
 
-                    if max_value == False:
+                    if max_value is False:
                         max_value = vector_color.max()
-                    if min_value == False:
+                    if min_value is False:
                         min_value = vector_color.min()
 
                     sp_vector = plot_axes.quiver(
@@ -1502,41 +1576,75 @@ class ImageVisualiser:
                         clim=[min_value, max_value],
                     )
 
-                if vector.show_colorbar:
+            if vector.show_colorbar:
 
-                    ax_box = ax_ax.get_position(original=False)
-                    cbar_width = ax_box.width * 0.05  # 5% of axis width
+                # divider = make_axes_locatable(ax_view)
+                # cax = divider.append_axes("right", size="5%", pad=0.05)
 
-                    if len(self.__figure.axes) >= 5:
-                        # There is a colorbar, add a new one
-
-                        cbar_axes = self.__figure.axes[4:]
-
-                        cbar_axes_label_pos_list = [
-                            self.__figure.transFigure.inverted().transform(
-                                i.yaxis.get_label().get_position()
-                            )[0]
-                            for i in cbar_axes
-                        ]
-
-                        max_xpos = max(cbar_axes_label_pos_list)
-
-                        x_pos_legend = max_xpos + 0.025
-
-                    else:
-                        x_pos_legend = ax_box.x1 + 0.025
+                ax_box = ax.get_position(original=False)
+                cbar_width = ax_box.width * 0.05  # 5% of axis width
+                cbar_color = "black"
 
+                if self.__axis == "ortho":
                     cax = self.__figure.add_axes(
                         (
-                            x_pos_legend,
+                            ax_box.x1 + 0.02 + (cbar_width + 0.1) * vector_index,
                             ax_box.y0 * 1.025,
                             cbar_width,
                             ax_box.height - ax_box.y0 * 0.05,
                         )
                     )
 
-                    cbar = self.__figure.colorbar(sp_vector, cax=cax, orientation="vertical")
-                    cbar.set_label(vector.name)
+                else:
+                    cax = self.__figure.add_axes(
+                        (
+                            ax_box.x1 - 0.02 - (cbar_width + 0.1) * (vector_index + 1),
+                            0.025,
+                            cbar_width,
+                            ax_box.height - ax_box.y1 * 0.05,
+                        )
+                    )
+
+                    # check background values
+                    if np.linalg.norm(colormap(0)[:3]) < 0.1:
+                        # background is dark
+                        cbar_color = "white"
+
+                cbar = self.__figure.colorbar(sp_vector, cax=cax, orientation="vertical")
+
+                # set color
+                cbar.outline.set_edgecolor(color=cbar_color)
+                cbar.ax.tick_params(color=cbar_color)
+                cax.tick_params(axis="x", colors=cbar_color)
+                cax.tick_params(axis="y", colors=cbar_color)
+
+                cbar.set_label(vector.name, color=cbar_color)
+                cbar.solids.set_alpha(1)
+
+                if vector.discrete_levels:
+
+                    if vector.mid_ticks:
+
+                        delta_tick = (max_value - min_value) / vector.discrete_levels
+                        cbar.set_ticks(
+                            np.linspace(
+                                min_value + delta_tick / 2,
+                                max_value - delta_tick / 2,
+                                vector.discrete_levels,
+                            )
+                        )
+                        cbar.set_ticklabels(
+                            np.linspace(min_value, max_value, vector.discrete_levels)
+                        )
+
+                    else:
+                        cbar.set_ticks(
+                            np.linspace(
+                                min_value,
+                                max_value,
+                                vector.discrete_levels + 1,
+                            )
+                        )
 
     def draw_bounding_box_on_axes(self, ax, view, box, add_label=True):
 
